"""
gmlp.evolution
==============
Your base module for starting your genetic programming.
"""
from __future__ import absolute_import

import configparser
import json
import os
import random
import re
import time
from io import BytesIO, StringIO

from .fitness import *
from .mutations import *
from .selection import *


class Error(Exception):
    pass


class SettingsError(Error):
    """Settings Not Defined!"""
    pass


class Enviroment:
    """
    Used to start your evolutionary neural network.

    :param goal: This is the goal for your evolutionary neural network. If your population is binary your goal should be binary.

    :param crossover_prob: The probability for your organisms to crossover.
    """

    def __init__(self, goal, crossover_prob):
        self.goal = goal
        self.crossover_prob = crossover_prob

    def generate_population(self, genes=None, pop_size=10000, binary=False):
        """
        Generates a population.

        :param genes: The genes per organism.

        :param pop_size: Your population size.

        :param binary: This is if you want your population in a binary form if True.
        """
        self.genes = genes
        self.size = pop_size
        self.binary = binary
        if self.binary == True:
            return [[random.randint(0, 1) for g in range(self.genes)]for n in range(self.size)]

    def crossover(self, population, target):
        """
        The crossover of your population.

        :param population: Your population.

        :param target: Your target.
        """
        self.pop = population
        self.target = target
        for k in range(int(len(self.pop)-2)):
            if random.random() < self.crossover_prob:
                self.parent1 = self.pop.pop(k)
                self.parent2 = self.pop.pop(k+1)
                self.crossover_point = random.randint(0, len(self.target))
                self.child1 = self.pop.insert(
                    k, self.parent1[0:self.crossover_point]+self.parent2[self.crossover_point:])
                self.child2 = self.pop.insert(
                    k+1, self.parent1[0:self.crossover_point]+self.parent2[self.crossover_point:])
        return self.pop

   

# class NeuralNetwork(Connection):
#     """
#     ``THIS FEATURE IS IN DEVELOPMENT!``
#     """
#     def __init__(self, **sett):
#         if sett in "inputs":
#             self.connect('Inputs', sett['inputs'])
        
#         if sett in "hidden":
#             self.connect('Hidden', sett['hidden'])

#         if sett in "output":
#             self.connect('Output', sett['output'])
        


# class Builder:
#     """
#     ``THIS FEATURE IS IN DEVELOPMENT``
#     For help on this module type Builder().help()
#     """
#     def __init__(self, settings=None):
#         self.settings = settings
#         if self.settings != None:
#             self.cfg = configparser.ConfigParser()
#             self.file = self.cfg.read(self.settings)
#             self.sections = self.cfg.sections()
#             for sect in range(len(self.sections)):
#                 if self.sections[sect] == 'NeuralNet':
#                     self.inputs = self.cfg.get(self.sections[sect], 'inputs')
#                     self.outputs = self.cfg.get(self.sections[sect], 'outputs')
#                     self.hidden = self.cfg.get(self.sections[sect], 'hidden')
                    
#                     NeuralNetwork(inputs=self.inputs, outputs=self.outputs, hidden=self.hidden).show()

#     def help(self):
#         self.info = """
#         The builder class is complicated so here's some help with the class.
#         For the settings param you need an .ini file. Heres an example:

#             Settings.ini

#             [NeuralNet]
#             # This is the params for our neural network.
#             inputs = 20
#             hidden = 1
#             output = 2
#             activation = sigmoid

#             [Evolution]
#             # Our evolutionary neural network params
#             pop_size = 10000
#             fittness_function = fit_func()


#         """
#         print(self.info)



        


"""
gmlp.fitness
============
Generates a fitness.
"""
from __future__ import absolute_import


def calculate_fitness(population, goal):
    """
    Calculate the fitness based on where you want to be from where you are.

    :param population: Your population which you'll be calculating the population.

    :param goal: Your goal that you want your fitness to be subtracted from to find the correct calculations.
    """
    fit_scores = []
    for organism in range(len(population)):
        fitness = 0
        place = population[organism]
        for p in range(len(goal)):
            fitness = fitness + abs(goal[p] - place[p])
            # where you want to be subtracted by where you are
        fit_scores.append(fitness)
    return fit_scores


# def custom_fitness(population, goal, starting_fit_value, PopulationHasGenes, GoalHasGenes):
#     scores = []
#     if PopulationHasGenes == True:
#         if GoalHasGenes == True:
#             for org in range(len(population)):
#                 fitness = starting_fit_value
#                 place = population[org]
#                 for target in range(len(goal)):
#                     goal_place = goal[target]
#                     fitness += abs(goal[goal_place] - place[target])
#                 scores.append(fitness)
#             return scores

#     elif PopulationHasGenes == False:
#         if GoalHasGenes == False:
#             for org in range(len(population)):
#                 fitness = starting_fit_value
#                 fitness = starting_fit_value
#                 fitness += abs(goal[org] - population[org])
#                 scores.append(fitness)
#             return scores

#     elif PopulationHasGenes == False:
#         if GoalHasGenes == True:
#             for org in range(len(population)):
#                 fitness = starting_fit_value
#                 for target in range(len(goal)):
#                     fitness += abs(goal[target] - population[org])
#                 scores.append(fitness)
#         return scores

#     elif PopulationHasGenes == True:
#         if GoalHasGenes == False:
#             for org in range(len(population)):
#                 fitness = starting_fit_value
#                 place = population[org]
#                 for target in range(len(goal)):
#                     fitness += abs(goal[target] - place[target])
#                 scores.append(fitness)
#             return scores



"""
GMLP.selection
"""
from __future__ import absolute_import

import random


def tournament_selection(population, scores, tournament_size):
    """
    Selects the fittest.

    :param population: This is your population param to be selected from.

    :param scores: Each of your population's fitness.

    :param tournament_size: How much organisms you want to select for each iteration.
    """
    population = population
    scores = scores
    tournament_size = tournament_size
    fittest = []
    for fit in range(0, len(population)):
        random_org = random.randint(0, len(scores)-1)
        prev_score = scores[random_org]
        prev_pop = population[random_org]
        for t in range(0, tournament_size):
            fighters = random.randint(0, len(scores)-1)
            if scores[fighters] < prev_score:
                prev_score = scores[fighters]
                prev_pop = population[fighters]
        fittest.append(prev_pop)
    return fittest



"""
gmlp.mutations
==============
Used for mutating your population. 
\n``More mutations are to come in future versions.``
"""
from __future__ import absolute_import

import random


def ValueEncodingMut(population, mutation_prob):
    """
    Value Encoding Mutation.

    :param population: the population you will be mutating. The have genes like this -> [[1, 2, 3], [4, 5, 6], [7, 8, 9]]

    :param mutation_prob: the mutation probability usually .15 but you'll have to see what's best for you.
    """
    for pop in range(len(population)):
        if random.random() < mutation_prob:
            random_population_org = random.randint(0, len(population)-1)
            popped_org = population.pop(random_population_org)
            for c in range(len(popped_org)):
                if random.random() < .5:
                    popped_org[c] += 1
                else:
                    popped_org[c] -= 1
            population.insert(random_population_org, popped_org)
    return population


def ShuffleMut(population, mutation_prob):
    """
    Shuffle mutation.

    :param population: Your selected population you wish to mutate.

    :param mutation_prob: The probability that your organisms will mutate.
    """
    if not population == list(population):
        population = list(population)
    for pop in range(len(population)):
        if random.random() < mutation_prob:
            rand_pop_place = random.randint(0, len(population)-1)
            random.shuffle(population[rand_pop_place])
    return population


def BinaryInvertMut(population, mutation_prob):
    """
    Inverts your binary population.

    :param population: Your selected population you wish to mutate.

    :param mutation_prob: The probability that your organisms will mutate.
    """
    for n in range(len(population)):
        if random.random() < mutation_prob:
            rand = random.randint(0, len(population)-1)
            if population[rand] == 0:
                population[rand] = 1
            else:
                population[rand] = 0
    return population




from setuptools import setup, find_packages

with open('README.md') as fh:
    long_description = fh.read()

setup(
    name='gmlp',  # Required
    version="0.2",
    description='Genetic Algorithm for Python',
    long_description=long_description, 
    long_description_content_type='text/markdown', 
    url='https://github.com/CoderWeird/gmlp',
    author='Drew Montooth',
    author_email='drewmontooth@gmail.com',
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7'
    ],

    packages=find_packages(),  # Required

    python_requires='>=2.7',

    install_requires=[
        'pygame'
    ],

    include_package_data=True
)